==> graphql-codegen-plugin-typescript-swr-0.5.1.tgz: package/tests/swr.spec.ts <==
import { Types, mergeOutputs } from '@graphql-codegen/plugin-helpers'
import { validateTs } from '@graphql-codegen/testing'
import {
  plugin as tsPlugin,
  TypeScriptPluginConfig,
} from '@graphql-codegen/typescript'
import { plugin as graphQLRequestPlugin } from '@graphql-codegen/typescript-graphql-request'
import { GraphQLRequestPluginConfig } from '@graphql-codegen/typescript-graphql-request/visitor'
import {
  plugin as tsDocumentsPlugin,
  TypeScriptDocumentsPluginConfig,
} from '@graphql-codegen/typescript-operations'
import { parse, GraphQLSchema, buildClientSchema } from 'graphql'

import { RawSWRPluginConfig } from '../src/config'
import { plugin } from '../src/index'

type PluginsConfig = Partial<
  TypeScriptPluginConfig &
    TypeScriptDocumentsPluginConfig &
    GraphQLRequestPluginConfig &
    RawSWRPluginConfig
>

describe('SWR', () => {
  const schema = buildClientSchema(require('../dev-test/githunt/schema.json'))

  const basicDoc = parse(/* GraphQL */ `
    query feed {
      feed {
        id
        commentCount
        repository {
          owner {
            avatar_url
          }
        }
      }
    }
    query feed2($v: String!) {
      feed {
        id
      }
    }
    query feed3($v: String) {
      feed {
        id
      }
    }
    query feed4($v: String! = "TEST") {
      feed {
        id
      }
    }
  `)

  const basicUsage = `
async function test() {
  const client = new GraphQLClient('');
  const sdk = getSdkWithHooks(client);

  await sdk.feed();
  await sdk.feed3();
  await sdk.feed4();
  const result = await sdk.feed2({ v: "1" });
  if (result.feed) {
    if (result.feed[0]) {
      const id = result.feed[0].id
    }
  }
}`

  const validate = async (
    content: Types.PluginOutput,
    config: PluginsConfig,
    docs: Types.DocumentFile[],
    pluginSchema: GraphQLSchema,
    usage: string
  ) => {
    const m = mergeOutputs([
      await tsPlugin(pluginSchema, docs, config, { outputFile: '' }),
      await tsDocumentsPlugin(pluginSchema, docs, config),
      await graphQLRequestPlugin(pluginSchema, docs, config),
      content,
      usage,
    ])

    await validateTs(m)

    return m
  }

  describe('sdk', () => {
    it('Should generate import declarations of output correctly', async () => {
      const config: PluginsConfig = {}
      const docs = [{ location: '', document: basicDoc }]
      const usage = ``

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      expect(content.prepend).toContain(
        `import useSWR, { ConfigInterface as SWRConfigInterface, keyInterface as SWRKeyInterface } from 'swr';`
      )

      await validate(content, config, docs, schema, usage)
    })

    it('Should support useTypeImports', async () => {
      const config: PluginsConfig = { useTypeImports: true }
      const docs = [{ location: '', document: basicDoc }]
      const usage = ``

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      expect(content.prepend).toContain(`import useSWR from 'swr';`)
      expect(content.prepend).toContain(
        `import type { ConfigInterface as SWRConfigInterface, keyInterface as SWRKeyInterface } from 'swr';`
      )

      await validate(content, config, docs, schema, usage)
    })
  })

  describe('plugin-typescript-swr', () => {
    it('Should generate Hooks API of output correctly', async () => {
      const config: PluginsConfig = {}
      const docs = [{ location: '', document: basicDoc }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(output).toContain(
        `export function getSdkWithHooks(client: GraphQLClient, withWrapper: SdkFunctionWrapper = defaultWrapper) {
  const sdk = getSdk(client, withWrapper);
  return {
    ...sdk,
    useFeed(key: SWRKeyInterface, variables?: FeedQueryVariables, config?: SWRConfigInterface<FeedQuery>) {
      return useSWR<FeedQuery>(key, () => sdk.feed(variables), config);
    },
    useFeed2(key: SWRKeyInterface, variables: Feed2QueryVariables, config?: SWRConfigInterface<Feed2Query>) {
      return useSWR<Feed2Query>(key, () => sdk.feed2(variables), config);
    },
    useFeed3(key: SWRKeyInterface, variables?: Feed3QueryVariables, config?: SWRConfigInterface<Feed3Query>) {
      return useSWR<Feed3Query>(key, () => sdk.feed3(variables), config);
    },
    useFeed4(key: SWRKeyInterface, variables?: Feed4QueryVariables, config?: SWRConfigInterface<Feed4Query>) {
      return useSWR<Feed4Query>(key, () => sdk.feed4(variables), config);
    }
  };
}
export type SdkWithHooks = ReturnType<typeof getSdkWithHooks>;`
      )
    })

    it('Should generate the output from which mutation operation has been removed', async () => {
      const config: PluginsConfig = {}
      const document = parse(/* GraphQL */ `
        query feed {
          feed {
            id
            commentCount
            repository {
              owner {
                avatar_url
              }
            }
          }
        }
        query feed2($v: String!) {
          feed {
            id
          }
        }
        query feed3($v: String) {
          feed {
            id
          }
        }
        query feed4($v: String! = "TEST") {
          feed {
            id
          }
        }
        mutation submitComment(
          $repoFullName: String!
          $commentContent: String!
        ) {
          submitComment(
            repoFullName: $repoFullName
            commentContent: $commentContent
          ) {
            ...CommentsPageComment
          }
        }
        fragment CommentsPageComment on Comment {
          id
          postedBy {
            login
            html_url
          }
          createdAt
          content
        }
      `)
      const docs = [{ location: '', document }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(output).toContain(
        `export function getSdkWithHooks(client: GraphQLClient, withWrapper: SdkFunctionWrapper = defaultWrapper) {
  const sdk = getSdk(client, withWrapper);
  return {
    ...sdk,
    useFeed(key: SWRKeyInterface, variables?: FeedQueryVariables, config?: SWRConfigInterface<FeedQuery>) {
      return useSWR<FeedQuery>(key, () => sdk.feed(variables), config);
    },
    useFeed2(key: SWRKeyInterface, variables: Feed2QueryVariables, config?: SWRConfigInterface<Feed2Query>) {
      return useSWR<Feed2Query>(key, () => sdk.feed2(variables), config);
    },
    useFeed3(key: SWRKeyInterface, variables?: Feed3QueryVariables, config?: SWRConfigInterface<Feed3Query>) {
      return useSWR<Feed3Query>(key, () => sdk.feed3(variables), config);
    },
    useFeed4(key: SWRKeyInterface, variables?: Feed4QueryVariables, config?: SWRConfigInterface<Feed4Query>) {
      return useSWR<Feed4Query>(key, () => sdk.feed4(variables), config);
    }
  };
}
export type SdkWithHooks = ReturnType<typeof getSdkWithHooks>;`
      )
    })

    it('Should work `excludeQueries` option correctly', async () => {
      const config: PluginsConfig = {
        excludeQueries: ['feed[2-3]', 'hoge', 'foo'],
      }
      const docs = [{ location: '', document: basicDoc }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(output).toContain(
        `export function getSdkWithHooks(client: GraphQLClient, withWrapper: SdkFunctionWrapper = defaultWrapper) {
  const sdk = getSdk(client, withWrapper);
  return {
    ...sdk,
    useFeed(key: SWRKeyInterface, variables?: FeedQueryVariables, config?: SWRConfigInterface<FeedQuery>) {
      return useSWR<FeedQuery>(key, () => sdk.feed(variables), config);
    },
    useFeed4(key: SWRKeyInterface, variables?: Feed4QueryVariables, config?: SWRConfigInterface<Feed4Query>) {
      return useSWR<Feed4Query>(key, () => sdk.feed4(variables), config);
    }
  };
}
export type SdkWithHooks = ReturnType<typeof getSdkWithHooks>;`
      )
    })

    it('Should work `useSWRInfinite` option correctly', async () => {
      const config: PluginsConfig = {
        useSWRInfinite: ['feed[24]'],
      }
      const docs = [{ location: '', document: basicDoc }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(content.prepend).toContain(
        `import useSWR, { useSWRInfinite, ConfigInterface as SWRConfigInterface, keyInterface as SWRKeyInterface, SWRInfiniteConfigInterface } from 'swr';`
      )
      expect(output).toContain(
        `export type SWRInfiniteKeyLoader<Data = unknown, Variables = unknown> = (
  index: number,
  previousPageData: Data | null
) => [keyof Variables, Variables[keyof Variables] | null] | null;
export function getSdkWithHooks(client: GraphQLClient, withWrapper: SdkFunctionWrapper = defaultWrapper) {
  const sdk = getSdk(client, withWrapper);
  const utilsForInfinite = {
    generateGetKey: <Data = unknown, Variables = unknown>(
      id: string,
      getKey: SWRInfiniteKeyLoader<Data, Variables>
    ) => (pageIndex: number, previousData: Data | null) => {
      const key = getKey(pageIndex, previousData)
      return key ? [id, ...key] : null
    },
    generateFetcher: <Query = unknown, Variables = unknown>(query: (variables: Variables) => Promise<Query>, variables?: Variables) => (
        id: string,
        fieldName: keyof Variables,
        fieldValue: Variables[typeof fieldName]
      ) => query({ ...variables, [fieldName]: fieldValue } as Variables)
  }
  return {
    ...sdk,
    useFeed(key: SWRKeyInterface, variables?: FeedQueryVariables, config?: SWRConfigInterface<FeedQuery>) {
      return useSWR<FeedQuery>(key, () => sdk.feed(variables), config);
    },
    useFeed2(key: SWRKeyInterface, variables: Feed2QueryVariables, config?: SWRConfigInterface<Feed2Query>) {
      return useSWR<Feed2Query>(key, () => sdk.feed2(variables), config);
    },
    useFeed2Infinite(id: string, getKey: SWRInfiniteKeyLoader<Feed2Query, Feed2QueryVariables>, variables: Feed2QueryVariables, config?: SWRInfiniteConfigInterface<Feed2Query>) {
      return useSWRInfinite<Feed2Query>(
        utilsForInfinite.generateGetKey<Feed2Query, Feed2QueryVariables>(id, getKey),
        utilsForInfinite.generateFetcher<Feed2Query, Feed2QueryVariables>(sdk.feed2, variables),
        config);
    },
    useFeed3(key: SWRKeyInterface, variables?: Feed3QueryVariables, config?: SWRConfigInterface<Feed3Query>) {
      return useSWR<Feed3Query>(key, () => sdk.feed3(variables), config);
    },
    useFeed4(key: SWRKeyInterface, variables?: Feed4QueryVariables, config?: SWRConfigInterface<Feed4Query>) {
      return useSWR<Feed4Query>(key, () => sdk.feed4(variables), config);
    },
    useFeed4Infinite(id: string, getKey: SWRInfiniteKeyLoader<Feed4Query, Feed4QueryVariables>, variables?: Feed4QueryVariables, config?: SWRInfiniteConfigInterface<Feed4Query>) {
      return useSWRInfinite<Feed4Query>(
        utilsForInfinite.generateGetKey<Feed4Query, Feed4QueryVariables>(id, getKey),
        utilsForInfinite.generateFetcher<Feed4Query, Feed4QueryVariables>(sdk.feed4, variables),
        config);
    }
  };
}
export type SdkWithHooks = ReturnType<typeof getSdkWithHooks>;`
      )
    })
  })

  it('Should work `autogenSWRKey` option correctly', async () => {
    const config: PluginsConfig = {
      autogenSWRKey: true,
    }
    const docs = [{ location: '', document: basicDoc }]

    const content = (await plugin(schema, docs, config, {
      outputFile: 'graphql.ts',
    })) as Types.ComplexPluginOutput

    const usage = basicUsage
    const output = await validate(content, config, docs, schema, usage)
    expect(output).toContain(
      `export function getSdkWithHooks(client: GraphQLClient, withWrapper: SdkFunctionWrapper = defaultWrapper) {
  const sdk = getSdk(client, withWrapper);
  const genKey = <V extends Record<string, unknown> = Record<string, unknown>>(name: string, object: V = {} as V): SWRKeyInterface => [name, ...Object.keys(object).sort().map(key => object[key])];
  return {
    ...sdk,
    useFeed(variables?: FeedQueryVariables, config?: SWRConfigInterface<FeedQuery>) {
      return useSWR<FeedQuery>(genKey<FeedQueryVariables>('Feed', variables), () => sdk.feed(variables), config);
    },
    useFeed2(variables: Feed2QueryVariables, config?: SWRConfigInterface<Feed2Query>) {
      return useSWR<Feed2Query>(genKey<Feed2QueryVariables>('Feed2', variables), () => sdk.feed2(variables), config);
    },
    useFeed3(variables?: Feed3QueryVariables, config?: SWRConfigInterface<Feed3Query>) {
      return useSWR<Feed3Query>(genKey<Feed3QueryVariables>('Feed3', variables), () => sdk.feed3(variables), config);
    },
    useFeed4(variables?: Feed4QueryVariables, config?: SWRConfigInterface<Feed4Query>) {
      return useSWR<Feed4Query>(genKey<Feed4QueryVariables>('Feed4', variables), () => sdk.feed4(variables), config);
    }
  };
}
export type SdkWithHooks = ReturnType<typeof getSdkWithHooks>;`
    )
  })
})

==> graphql-codegen-plugin-typescript-swr-0.5.2.tgz: package/tests/swr.spec.ts <==
import fs from 'fs'
import { resolve } from 'path'

import { Types, mergeOutputs } from '@graphql-codegen/plugin-helpers'
import { validateTs } from '@graphql-codegen/testing'
import {
  plugin as tsPlugin,
  TypeScriptPluginConfig,
} from '@graphql-codegen/typescript'
import { plugin as graphQLRequestPlugin } from '@graphql-codegen/typescript-graphql-request'
import { GraphQLRequestPluginConfig } from '@graphql-codegen/typescript-graphql-request/visitor'
import {
  plugin as tsDocumentsPlugin,
  TypeScriptDocumentsPluginConfig,
} from '@graphql-codegen/typescript-operations'
import { parse, GraphQLSchema, buildClientSchema } from 'graphql'

import { RawSWRPluginConfig } from '../src/config'
import { plugin } from '../src/index'

type PluginsConfig = Partial<
  TypeScriptPluginConfig &
    TypeScriptDocumentsPluginConfig &
    GraphQLRequestPluginConfig &
    RawSWRPluginConfig
>

const readOutput = (name: string): string => {
  return fs.readFileSync(resolve(__dirname, `./outputs/${name}.ts`), 'utf-8')
}

describe('SWR', () => {
  const schema = buildClientSchema(require('../dev-test/githunt/schema.json'))

  const basicDoc = parse(/* GraphQL */ `
    query feed {
      feed {
        id
        commentCount
        repository {
          owner {
            avatar_url
          }
        }
      }
    }
    query feed2($v: String!) {
      feed {
        id
      }
    }
    query feed3($v: String) {
      feed {
        id
      }
    }
    query feed4($v: String! = "TEST") {
      feed {
        id
      }
    }
  `)

  const basicUsage = `
async function test() {
  const client = new GraphQLClient('');
  const sdk = getSdkWithHooks(client);

  await sdk.feed();
  await sdk.feed3();
  await sdk.feed4();
  const result = await sdk.feed2({ v: "1" });
  if (result.feed) {
    if (result.feed[0]) {
      const id = result.feed[0].id
    }
  }
}`

  const validate = async (
    content: Types.PluginOutput,
    config: PluginsConfig,
    docs: Types.DocumentFile[],
    pluginSchema: GraphQLSchema,
    usage: string
  ) => {
    const m = mergeOutputs([
      await tsPlugin(pluginSchema, docs, config, { outputFile: '' }),
      await tsDocumentsPlugin(pluginSchema, docs, config),
      await graphQLRequestPlugin(pluginSchema, docs, config),
      content,
      usage,
    ])

    await validateTs(m)

    return m
  }

  describe('sdk', () => {
    it('Should generate import declarations of output correctly', async () => {
      const config: PluginsConfig = {}
      const docs = [{ location: '', document: basicDoc }]
      const usage = ``

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      expect(content.prepend).toContain(
        `import useSWR, { ConfigInterface as SWRConfigInterface, keyInterface as SWRKeyInterface } from 'swr';`
      )

      await validate(content, config, docs, schema, usage)
    })

    it('Should support useTypeImports', async () => {
      const config: PluginsConfig = { useTypeImports: true }
      const docs = [{ location: '', document: basicDoc }]
      const usage = ``

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      expect(content.prepend).toContain(`import useSWR from 'swr';`)
      expect(content.prepend).toContain(
        `import type { ConfigInterface as SWRConfigInterface, keyInterface as SWRKeyInterface } from 'swr';`
      )

      await validate(content, config, docs, schema, usage)
    })
  })

  describe('plugin-typescript-swr', () => {
    it('Should generate Hooks API of output correctly', async () => {
      const config: PluginsConfig = {}
      const docs = [{ location: '', document: basicDoc }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(output).toContain(readOutput('straight'))
    })

    it('Should generate the output from which mutation operation has been removed', async () => {
      const config: PluginsConfig = {}
      const document = parse(/* GraphQL */ `
        query feed {
          feed {
            id
            commentCount
            repository {
              owner {
                avatar_url
              }
            }
          }
        }
        query feed2($v: String!) {
          feed {
            id
          }
        }
        query feed3($v: String) {
          feed {
            id
          }
        }
        query feed4($v: String! = "TEST") {
          feed {
            id
          }
        }
        mutation submitComment(
          $repoFullName: String!
          $commentContent: String!
        ) {
          submitComment(
            repoFullName: $repoFullName
            commentContent: $commentContent
          ) {
            ...CommentsPageComment
          }
        }
        fragment CommentsPageComment on Comment {
          id
          postedBy {
            login
            html_url
          }
          createdAt
          content
        }
      `)
      const docs = [{ location: '', document }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(output).toContain(readOutput('mutations'))
    })

    it('Should work `excludeQueries` option correctly', async () => {
      const config: PluginsConfig = {
        excludeQueries: ['feed[2-3]', 'hoge', 'foo'],
      }
      const docs = [{ location: '', document: basicDoc }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(output).toContain(readOutput('excludeQueries'))
    })

    it('Should work `useSWRInfinite` option correctly', async () => {
      const config: PluginsConfig = {
        useSWRInfinite: ['feed[24]'],
      }
      const docs = [{ location: '', document: basicDoc }]

      const content = (await plugin(schema, docs, config, {
        outputFile: 'graphql.ts',
      })) as Types.ComplexPluginOutput

      const usage = basicUsage
      const output = await validate(content, config, docs, schema, usage)
      expect(content.prepend).toContain(
        `import useSWR, { useSWRInfinite, ConfigInterface as SWRConfigInterface, keyInterface as SWRKeyInterface, SWRInfiniteConfigInterface } from 'swr';`
      )
      expect(output).toContain(readOutput('infinite'))
    })
  })

  it('Should work `autogenSWRKey` option correctly', async () => {
    const config: PluginsConfig = {
      autogenSWRKey: true,
      useSWRInfinite: ['feed[24]'],
    }
    const docs = [{ location: '', document: basicDoc }]

    const content = (await plugin(schema, docs, config, {
      outputFile: 'graphql.ts',
    })) as Types.ComplexPluginOutput

    const usage = basicUsage
    const output = await validate(content, config, docs, schema, usage)
    expect(output).toContain(readOutput('autogenSWRKey'))
  })
})
